@@ -1,125 +1,121 @@
 #! /usr/local/perl -w
 package main;
 require Test::Harness;
-*Verbose = \$Test::Harness::Verbose;
 use Data::Dumper;
-use POSIX qw/locale_h/;
 use File::Temp qw/tempfile/;
 use File::Basename;
 
+if ($Test::More::VERSION < 0.48) { # Fix for RT#48268
+    local $^W;
+    *main::use_ok = sub ($;@) {
+	my ($pkg, $req, @args) = @_;
+	eval "use $pkg $req ".join(' ',@args);
+	is ${"$pkg\::VERSION"}, eval($req), 'Had to manually use version';
+	# If we made it this far, we are ok.
+    };
+}
+
 sub BaseTests {
 
     my ($CLASS, $method, $qv_declare) = @_;
     my $warning;
     local $SIG{__WARN__} = sub { $warning = $_[0] };
-    
+
     # Insert your test code below, the Test module is use()ed here so read
     # its man page ( perldoc Test ) for help writing this test script.
-    
+
     # Test bare number processing
-    diag "tests with bare numbers" if $Verbose;
     $version = $CLASS->$method(5.005_03);
     is ( "$version" , "5.00503" , '5.005_03 eq 5.00503' );
     $version = $CLASS->$method(1.23);
     is ( "$version" , "1.23" , '1.23 eq "1.23"' );
-    
+
+    # Test explicit integer
+    $version = $CLASS->$method(23);
+    is ( "$version" , 23 , '23 eq "23"' );
+
     # Test quoted number processing
-    diag "tests with quoted numbers" if $Verbose;
     $version = $CLASS->$method("5.005_03");
     is ( "$version" , "5.005_03" , '"5.005_03" eq "5.005_03"' );
     $version = $CLASS->$method("v1.23");
     is ( "$version" , "v1.23" , '"v1.23" eq "v1.23"' );
-    
+
     # Test stringify operator
-    diag "tests with stringify" if $Verbose;
     $version = $CLASS->$method("5.005");
     is ( "$version" , "5.005" , '5.005 eq "5.005"' );
     $version = $CLASS->$method("5.006.001");
     is ( "$version" , "5.006.001" , '5.006.001 eq v5.6.1' );
-    like ($warning, qr/v-string without leading 'v' deprecated/, 'No leading v');
+    unlike ($warning, qr/v-string without leading 'v' deprecated/, 'No leading v');
     $version = $CLASS->$method("v1.2.3_4");
     is ( "$version" , "v1.2.3_4" , 'alpha version 1.2.3_4 eq v1.2.3_4' );
-    
+
     # test illegal formats
-    diag "test illegal formats" if $Verbose;
     eval {my $version = $CLASS->$method("1.2_3_4")};
     like($@, qr/multiple underscores/,
 	"Invalid version format (multiple underscores)");
-    
+
     eval {my $version = $CLASS->$method("1.2_3.4")};
     like($@, qr/underscores before decimal/,
 	"Invalid version format (underscores before decimal)");
-    
+
     eval {my $version = $CLASS->$method("1_2")};
     like($@, qr/alpha without decimal/,
 	"Invalid version format (alpha without decimal)");
-    
-    # for this test, upgrade the warn() to die()
-    eval {
-	local $SIG{__WARN__} = sub { die $_[0] };
-	$version = $CLASS->$method("1.2b3");
-    };
-    my $warnregex = "Version string '.+' contains invalid data; ".
-	    "ignoring: '.+'";
 
-    like($@, qr/$warnregex/,
-	"Version string contains invalid data; ignoring");
+    eval { $version = $CLASS->$method("1.2b3")};
+    like($@, qr/non-numeric data/,
+	"Invalid version format (non-numeric data)");
+
+    eval { $version = $CLASS->$method("-1.23")};
+    like($@, qr/negative version number/,
+	"Invalid version format (negative version number)");
 
     # from here on out capture the warning and test independently
     {
-    $version = $CLASS->$method("99 and 44/100 pure");
+    eval{$version = $CLASS->$method("99 and 44/100 pure")};
+
+    like($@, qr/non-numeric data/,
+	"Invalid version format (non-numeric data)");
+
+    eval{$version = $CLASS->$method("something")};
+    like($@, qr/non-numeric data/,
+	"Invalid version format (non-numeric data)");
 
-    like($warning, qr/$warnregex/,
-	"Version string contains invalid data; ignoring");
-    is ("$version", "99", '$version eq "99"');
-    ok ($version->numify == 99.0, '$version->numify == 99.0');
-    ok ($version->normal eq "v99.0.0", '$version->normal eq v99.0.0');
-    
-    $version = $CLASS->$method("something");
-    like($warning, qr/$warnregex/,
-	"Version string contains invalid data; ignoring");
-    ok (defined $version, 'defined $version');
-    
     # reset the test object to something reasonable
     $version = $CLASS->$method("1.2.3");
-    
+
     # Test boolean operator
     ok ($version, 'boolean');
-    
+
     # Test class membership
     isa_ok ( $version, $CLASS );
-    
+
     # Test comparison operators with self
-    diag "tests with self" if $Verbose;
     is ( $version <=> $version, 0, '$version <=> $version == 0' );
     ok ( $version == $version, '$version == $version' );
-    
+
     # Test Numeric Comparison operators
     # test first with non-object
     $version = $CLASS->$method("5.006.001");
     $new_version = "5.8.0";
-    diag "numeric tests with non-objects" if $Verbose;
     ok ( $version == $version, '$version == $version' );
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
-    
+
     # now test with existing object
     $new_version = $CLASS->$method($new_version);
-    diag "numeric tests with objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
-    
+
     # now test with actual numbers
-    diag "numeric tests with numbers" if $Verbose;
     ok ( $version->numify() == 5.006001, '$version->numify() == 5.006001' );
     ok ( $version->numify() <= 5.006001, '$version->numify() <= 5.006001' );
     ok ( $version->numify() < 5.008, '$version->numify() < 5.008' );
     #ok ( $version->numify() > v5.005_02, '$version->numify() > 5.005_02' );
-    
+
     # test with long decimals
-    diag "Tests with extended decimal versions" if $Verbose;
     $version = $CLASS->$method(1.002003);
     ok ( $version == "1.2.3", '$version == "1.2.3"');
     ok ( $version->numify == 1.002003, '$version->numify == 1.002003');
@@ -127,61 +123,57 @@ sub BaseTests {
     ok ( $version == "2002.9.30.1",'$version == 2002.9.30.1');
     ok ( $version->numify == 2002.009030001,
 	'$version->numify == 2002.009030001');
-    
+
     # now test with alpha version form with string
     $version = $CLASS->$method("1.2.3");
     $new_version = "1.2.3_4";
-    diag "numeric tests with alpha-style non-objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
-    
+
     $version = $CLASS->$method("1.2.4");
-    diag "numeric tests with alpha-style non-objects"
-	if $Verbose;
-    ok ( $version > $new_version, '$version > $new_version' );
-    ok ( $new_version < $version, '$new_version < $version' );
+    ok ( $version < $new_version, '$version < $new_version' );
+    ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
-    
+
     # now test with alpha version form with object
     $version = $CLASS->$method("1.2.3");
     $new_version = $CLASS->$method("1.2.3_4");
-    diag "tests with alpha-style objects" if $Verbose;
     ok ( $version < $new_version, '$version < $new_version' );
     ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
     ok ( !$version->is_alpha, '!$version->is_alpha');
     ok ( $new_version->is_alpha, '$new_version->is_alpha');
-    
+
     $version = $CLASS->$method("1.2.4");
-    diag "tests with alpha-style objects" if $Verbose;
-    ok ( $version > $new_version, '$version > $new_version' );
-    ok ( $new_version < $version, '$new_version < $version' );
+    ok ( $version < $new_version, '$version < $new_version' );
+    ok ( $new_version > $version, '$new_version > $version' );
     ok ( $version != $new_version, '$version != $new_version' );
-    
-    $version = $CLASS->$method("1.2.3.4");
+
+    $version = $CLASS->$method("1.2.34");
     $new_version = $CLASS->$method("1.2.3_4");
-    diag "tests with alpha-style objects with same subversion"
-	if $Verbose;
-    ok ( $version > $new_version, '$version > $new_version' );
-    ok ( $new_version < $version, '$new_version < $version' );
-    ok ( $version != $new_version, '$version != $new_version' );
-    
-    diag "test implicit [in]equality" if $Verbose;
-    $version = $CLASS->$method("v1.2.3");
-    $new_version = $CLASS->$method("1.2.3.0");
+    ok ( $version == $new_version, '$version == $new_version' );
+
+    $version = $CLASS->$method("v1.2.30");
+    $new_version = $CLASS->$method("1.2.30.0");
     ok ( $version == $new_version, '$version == $new_version' );
     $new_version = $CLASS->$method("1.2.3_0");
     ok ( $version == $new_version, '$version == $new_version' );
-    $new_version = $CLASS->$method("1.2.3.1");
+    $new_version = $CLASS->$method("1.2.30.1");
     ok ( $version < $new_version, '$version < $new_version' );
-    $new_version = $CLASS->$method("1.2.3_1");
+    $new_version = $CLASS->$method("1.2.30_1");
     ok ( $version < $new_version, '$version < $new_version' );
     $new_version = $CLASS->$method("1.1.999");
     ok ( $version > $new_version, '$version > $new_version' );
-    
+
+    $version = $CLASS->$method("v1.2.3");
+    eval { () = $version < 'version' };
+    # this test, and only this test, I have to do this or else $@ gets
+    # "reset" before like() has a chance to evaluate it.  Quite maddening!!!
+    my $err = $@;
+    like $err, qr/^Invalid version format/, "error with $version < 'version'";
+
     # that which is not expressly permitted is forbidden
-    diag "forbidden operations" if $Verbose;
     ok ( !eval { ++$version }, "noop ++" );
     ok ( !eval { --$version }, "noop --" );
     ok ( !eval { $version/1 }, "noop /" );
@@ -190,9 +182,8 @@ sub BaseTests {
 
 SKIP: {
     skip "version require'd instead of use'd, cannot test $qv_declare", 3
-    	unless defined $qv_declare;
+	unless defined $qv_declare;
     # test the $qv_declare() sub
-    diag "testing $qv_declare" if $Verbose;
     $version = $CLASS->$qv_declare("1.2");
     is ( "$version", "v1.2", $qv_declare.'("1.2") == "1.2.0"' );
     $version = $CLASS->$qv_declare(1.2);
@@ -201,37 +192,33 @@ SKIP: {
 }
 
     # test creation from existing version object
-    diag "create new from existing version" if $Verbose;
     ok (eval {$new_version = $CLASS->$method($version)},
 	    "new from existing object");
     ok ($new_version == $version, "class->$method($version) identical");
-    $new_version = $version->$method();
+    $new_version = $version->$method(0);
     isa_ok ($new_version, $CLASS );
     is ($new_version, "0", "version->$method() doesn't clone");
     $new_version = $version->$method("1.2.3");
     is ($new_version, "1.2.3" , '$version->$method("1.2.3") works too');
 
     # test the CVS revision mode
-    diag "testing CVS Revision" if $Verbose;
     $version = new $CLASS qw$Revision: 1.2$;
     ok ( $version == "1.2.0", 'qw$Revision: 1.2$ == 1.2.0' );
     $version = new $CLASS qw$Revision: 1.2.3.4$;
     ok ( $version == "1.2.3.4", 'qw$Revision: 1.2.3.4$ == 1.2.3.4' );
-    
+
     # test the CPAN style reduced significant digit form
-    diag "testing CPAN-style versions" if $Verbose;
     $version = $CLASS->$method("1.23_01");
     is ( "$version" , "1.23_01", "CPAN-style alpha version" );
     ok ( $version > 1.23, "1.23_01 > 1.23");
     ok ( $version < 1.24, "1.23_01 < 1.24");
 
     # test reformed UNIVERSAL::VERSION
-    diag "Replacement UNIVERSAL::VERSION tests" if $Verbose;
 
     my $error_regex = $] < 5.006
 	? 'version \d required'
 	: 'does not define \$t.{7}::VERSION';
-    
+
     {
 	my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
 	(my $package = basename($filename)) =~ s/\.pm$//;
@@ -242,7 +229,7 @@ SKIP: {
 	eval "use lib '.'; use $package $version";
 	unlike($@, qr/$package version $version/,
 		'Replacement eval works with exact version');
-	
+
 	# test as class method
 	$new_version = $package->VERSION;
 	cmp_ok($new_version,'==',$version, "Called as class method");
@@ -262,13 +249,13 @@ SKIP: {
 	eval "use lib '.'; use $package $version";
 	like($@, qr/$package version $version/,
 		'Replacement eval works with incremented version');
-	
+
 	$version =~ s/0+$//; #convert to string and remove trailing 0's
 	chop($version);	# shorten by 1 digit, should still succeed
 	eval "use lib '.'; use $package $version";
 	unlike($@, qr/$package version $version/,
 		'Replacement eval works with single digit');
-	
+
 	# this would fail with old UNIVERSAL::VERSION
 	$version += 0.1;
 	eval "use lib '.'; use $package $version";
@@ -286,19 +273,19 @@ SKIP: {
 	eval "use lib '.'; use $package 3;";
 	if ( $] < 5.008 ) {
 	    like($@, qr/$error_regex/,
-		'Replacement handles modules without package or VERSION'); 
+		'Replacement handles modules without package or VERSION');
 	}
 	else {
 	    like($@, qr/defines neither package nor VERSION/,
-		'Replacement handles modules without package or VERSION'); 
+		'Replacement handles modules without package or VERSION');
 	}
 	eval "use lib '.'; use $package; \$version = $package->VERSION";
 	unlike ($@, qr/$error_regex/,
-	    'Replacement handles modules without package or VERSION'); 
+	    'Replacement handles modules without package or VERSION');
 	ok (!defined($version), "Called as class method");
 	unlink $filename;
     }
-    
+
     { # dummy up some variously broken modules for testing
 	my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
 	(my $package = basename($filename)) =~ s/\.pm$//;
@@ -306,10 +293,10 @@ SKIP: {
 	close $fh;
 	eval "use lib '.'; use $package 3;";
 	like ($@, qr/$error_regex/,
-	    'Replacement handles modules without VERSION'); 
+	    'Replacement handles modules without VERSION');
 	eval "use lib '.'; use $package; print $package->VERSION";
 	unlike ($@, qr/$error_regex/,
-	    'Replacement handles modules without VERSION'); 
+	    'Replacement handles modules without VERSION');
 	unlink $filename;
     }
 
@@ -320,29 +307,50 @@ SKIP: {
 	close $fh;
 	eval "use lib '.'; use $package 3;";
 	like ($@, qr/$error_regex/,
-	    'Replacement handles modules without VERSION'); 
+	    'Replacement handles modules without VERSION');
 	eval "use lib '.'; use $package; print $package->VERSION";
 	unlike ($@, qr/$error_regex/,
-	    'Replacement handles modules without VERSION'); 
+	    'Replacement handles modules without VERSION');
 	unlink $filename;
     }
+SKIP:    { # https://rt.perl.org/rt3/Ticket/Display.html?id=95544
+	skip "version require'd instead of use'd, cannot test UNIVERSAL::VERSION", 2
+	    unless defined $qv_declare;
+	my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
+	(my $package = basename($filename)) =~ s/\.pm$//;
+	print $fh "package $package;\n\$VERSION = '3alpha';\n1;\n";
+	close $fh;
+	eval "use lib '.'; use $package; print $package->VERSION";
+	like ($@, qr/Invalid version format \(non-numeric data\)/,
+	    'Warn about bad \$VERSION');
+	eval "use lib '.'; use $package 1;";
+	like ($@, qr/Invalid version format \(non-numeric data\)/,
+	    'Warn about bad $VERSION');
+    }
 
 SKIP: 	{
 	skip 'Cannot test bare v-strings with Perl < 5.6.0', 4
-		if $] < 5.006_000; 
-	diag "Tests with v-strings" if $Verbose;
+		if $] < 5.006_000;
 	$version = $CLASS->$method(1.2.3);
-	ok("$version" == "v1.2.3", '"$version" == 1.2.3');
+	ok("$version" eq "v1.2.3", '"$version" eq 1.2.3');
 	$version = $CLASS->$method(1.0.0);
 	$new_version = $CLASS->$method(1);
 	ok($version == $new_version, '$version == $new_version');
 	skip "version require'd instead of use'd, cannot test declare", 1
 	    unless defined $qv_declare;
 	$version = &$qv_declare(1.2.3);
-	ok("$version" == "v1.2.3", 'v-string initialized $qv_declare()');
+	ok("$version" eq "v1.2.3", 'v-string initialized $qv_declare()');
     }
 
-    diag "Tests with real-world (malformed) data" if $Verbose;
+SKIP: 	{
+	skip 'Cannot test bare alpha v-strings with Perl < 5.8.1', 2
+		if $] lt 5.008_001;
+	$version = $CLASS->$method(v1.2.3_4);
+	$DB::single = 1;
+	is($version, "v1.2.3_4", '"$version" eq "v1.2.3_4"');
+	$version = $CLASS->$method(eval "v1.2.3_4");
+	is($version, "v1.2.3_4", '"$version" eq "v1.2.3_4" (from eval)');
+    }
 
     # trailing zero testing (reported by Andreas Koenig).
     $version = $CLASS->$method("1");
@@ -353,7 +361,7 @@ SKIP: 	{
     ok($version->numify eq "1.000000", "trailing zeros preserved");
     $version = $CLASS->$method("1.0.0.0");
     ok($version->numify eq "1.000000000", "trailing zeros preserved");
-    
+
     # leading zero testing (reported by Andreas Koenig).
     $version = $CLASS->$method(".7");
     ok($version->numify eq "0.700", "leading zero inferred");
@@ -381,7 +389,7 @@ SKIP: 	{
 
     $version = $CLASS->$method(0.000001);
     unlike($warning, qr/^Version string '1e-06' contains invalid data/,
-    	"Very small version objects");
+	"Very small version objects");
     }
 
 SKIP: {
@@ -399,19 +407,19 @@ EOF
 
 	eval "use lib '.'; use $package 0.000008;";
 	like ($@, qr/^$package version 0.000008 required/,
-	    "Make sure very small versions don't freak"); 
+	    "Make sure very small versions don't freak");
 	eval "use lib '.'; use $package 1;";
 	like ($@, qr/^$package version 1 required/,
-	    "Comparing vs. version with no decimal"); 
+	    "Comparing vs. version with no decimal");
 	eval "use lib '.'; use $package 1.;";
 	like ($@, qr/^$package version 1 required/,
-	    "Comparing vs. version with decimal only"); 
+	    "Comparing vs. version with decimal only");
 	if ( $] < 5.006_000 ) {
-	    skip 'Cannot "use" extended versions with Perl < 5.6.0', 3; 
+	    skip 'Cannot "use" extended versions with Perl < 5.6.0', 3;
 	}
 	eval "use lib '.'; use $package v0.0.8;";
 	my $regex = "^$package version v0.0.8 required";
-	like ($@, qr/$regex/, "Make sure very small versions don't freak"); 
+	like ($@, qr/$regex/, "Make sure very small versions don't freak");
 
 	$regex =~ s/8/4/; # set for second test
 	eval "use lib '.'; use $package v0.0.4;";
@@ -421,13 +429,13 @@ EOF
     }
 
 SKIP: {
-    skip 'Cannot test "use base qw(version)"  when require is used', 3
-    	unless defined $qv_declare;
+    skip "Cannot test \"use parent $CLASS\"  when require is used", 3
+	unless defined $qv_declare;
     my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
     (my $package = basename($filename)) =~ s/\.pm$//;
     print $fh <<"EOF";
 package $package;
-use base qw(version);
+use parent $CLASS;
 1;
 EOF
     close $fh;
@@ -442,7 +450,7 @@ EOF
 
 SKIP: {
 	if ( $] < 5.006_000 ) {
-	    skip 'Cannot "use" extended versions with Perl < 5.6.0', 3; 
+	    skip 'Cannot "use" extended versions with Perl < 5.6.0', 3;
 	}
 	my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
 	(my $package = basename($filename)) =~ s/\.pm$//;
@@ -454,70 +462,16 @@ EOF
 	close $fh;
 	eval "use lib '.'; use $package 1.001;";
 	like ($@, qr/^$package version 1.001 required/,
-	    "User typed numeric so we error with numeric"); 
+	    "User typed numeric so we error with numeric");
 	eval "use lib '.'; use $package v1.1.0;";
 	like ($@, qr/^$package version v1.1.0 required/,
-	    "User typed extended so we error with extended"); 
+	    "User typed extended so we error with extended");
 	unlink $filename;
     }
 
-SKIP: {
-	# test locale handling
-	my $warning;
-	local $SIG{__WARN__} = sub { $warning = $_[0] };
-
-$DB::single = 1;
-	my $v = $CLASS->$method('1,7');
-	unlike($warning, qr"Version string '1,7' contains invalid data",
-	    'Directly test comma as decimal compliance');
-
-	my $ver = 1.23;  # has to be floating point number
-	my $orig_loc = setlocale( LC_ALL );
-	my $loc;
-	while (<DATA>) {
-	    chomp;
-	    $loc = setlocale( LC_ALL, $_);
-	    last if localeconv()->{decimal_point} eq ',';
-	}
-	skip 'Cannot test locale handling without a comma locale', 4
-	    unless ( $loc and ($ver eq '1,23') );
-
-	diag ("Testing locale handling with $loc") if $Verbose;
-
-	$v = $CLASS->$method($ver);
-	unlike($warning, qr/Version string '1,23' contains invalid data/,
-	    "Process locale-dependent floating point");
-	is ($v, "1.23", "Locale doesn't apply to version objects");
-	ok ($v == $ver, "Comparison to locale floating point");
-
-	setlocale( LC_ALL, $orig_loc); # reset this before possible skip
-	skip 'Cannot test RT#46921 with Perl < 5.008', 1
-	    if ($] < 5.008);
-	skip 'Cannot test RT#46921 with pure Perl module', 1
-	    if exists $INC{'version/vpp.pm'};
-	my ($fh, $filename) = tempfile('tXXXXXXX', SUFFIX => '.pm', UNLINK => 1);
-	(my $package = basename($filename)) =~ s/\.pm$//;
-	print $fh <<"EOF";
-package $package;
-use POSIX qw(locale_h);
-\$^W = 1;
-use $CLASS;
-setlocale (LC_ALL, '$loc');
-use $CLASS ;
-eval "use Socket 1.7";
-setlocale( LC_ALL, '$orig_loc');
-1;
-EOF
-	close $fh;
-
-	eval "use lib '.'; use $package;";
-	unlike($warning, qr"Version string '1,7' contains invalid data",
-	    'Handle locale action-at-a-distance');
-    }
-
     eval 'my $v = $CLASS->$method("1._1");';
     unlike($@, qr/^Invalid version format \(alpha with zero width\)/,
-    	"Invalid version format 1._1");
+	"Invalid version format 1._1");
 
     {
 	my $warning;
@@ -528,6 +482,7 @@ EOF
     }
 
     {
+        local $Data::Dumper::Sortkeys= 1;
 	# http://rt.cpan.org/Public/Bug/Display.html?id=30004
 	my $v1 = $CLASS->$method("v0.1_1");
 	(my $alpha1 = Dumper($v1)) =~ s/.+'alpha' => ([^,]+),.+/$1/ms;
@@ -538,182 +493,147 @@ EOF
 
     {
 	# http://rt.perl.org/rt3/Ticket/Display.html?id=56606
-	my $badv = bless { version => [1,2,3] }, "version";
-	is $badv, '1.002003', "Deal with badly serialized versions from YAML";	
-	my $badv2 = bless { qv => 1, version => [1,2,3] }, "version";
-	is $badv2, 'v1.2.3', "Deal with badly serialized versions from YAML ";	
+	my $badv = bless { version => [1,2,3] }, $CLASS;
+	is $badv, '1.002003', "Deal with badly serialized versions from YAML";
+	my $badv2 = bless { qv => 1, version => [1,2,3] }, $CLASS;
+	is $badv2, 'v1.2.3', "Deal with badly serialized versions from YAML ";
+    }
+
+    {
+	# https://rt.cpan.org/Public/Bug/Display.html?id=70950
+	# test indirect usage of version objects
+	my $sum = 0;
+	eval '$sum += $CLASS->$method("v2.0.0")';
+	like $@, qr/operation not supported with version object/,
+	    'No math operations with version objects';
+	# test direct usage of version objects
+	my $v = $CLASS->$method("v2.0.0");
+	eval '$v += 1';
+	like $@, qr/operation not supported with version object/,
+	    'No math operations with version objects';
+    }
+
+    {
+	# https://rt.cpan.org/Ticket/Display.html?id=72365
+	# https://rt.perl.org/rt3/Ticket/Display.html?id=102586
+    	# https://rt.cpan.org/Ticket/Display.html?id=78328
+	eval 'my $v = $CLASS->$method("version")';
+	like $@, qr/Invalid version format/,
+	    "The string 'version' is not a version for $method";
+	eval 'my $v = $CLASS->$method("ver510n")';
+	like $@, qr/Invalid version format/,
+	    'All strings starting with "v" are not versions';
+    }
+
+SKIP: {
+	if ( $] < 5.006_000 ) {
+	    skip 'No v-string support at all < 5.6.0', 2;
+	}
+	# https://rt.cpan.org/Ticket/Display.html?id=49348
+	my $v = $CLASS->$method("420");
+	is "$v", "420", 'Correctly guesses this is not a v-string';
+	$v = $CLASS->$method(4.2.0);
+	is "$v", 'v4.2.0', 'Correctly guess that this is a v-string';
+    }
+SKIP: {
+	if ( $] < 5.006_000 ) {
+	    skip 'No v-string support at all < 5.6.0', 4;
+	}
+	# https://rt.cpan.org/Ticket/Display.html?id=50347
+	# Check that the qv() implementation does not change
+
+	ok $CLASS->$method(1.2.3) < $CLASS->$method(1.2.3.1), 'Compare 3 and 4 digit v-strings' ;
+	ok $CLASS->$method(v1.2.3) < $CLASS->$method(v1.2.3.1), 'Compare 3 and 4 digit v-strings, leaving v';
+	ok $CLASS->$method("1.2.3") < $CLASS->$method("1.2.3.1"), 'Compare 3 and 4 digit v-strings, quoted';
+	ok $CLASS->$method("v1.2.3") < $CLASS->$method("v1.2.3.1"), 'Compare 3 and 4 digit v-strings, quoted leading v';
+    }
+
+    {
+	eval '$CLASS->$method("version")';
+	pass("no crash with ${CLASS}->${method}('version')");
+	{
+	    package _102586;
+	    sub TIESCALAR { bless [] }
+	    sub FETCH { "version" }
+	    sub STORE { }
+	    my $v;
+	    tie $v, __PACKAGE__;
+	    $v = $CLASS->$method(1);
+	    eval '$CLASS->$method($v)';
+	}
+	pass('no crash with version->new($tied) where $tied returns "version"');
+    }
+
+    { # [perl #112478]
+	$_112478::VERSION = 9e99;
+	ok eval { _112478->VERSION(9e99); 1 }, '->VERSION(9e99) succeeds'
+	    or diag $@;
+	$_112478::VERSION = 1;
+	eval { _112478->VERSION(9e99) };
+	unlike $@, qr/panic/, '->VERSION(9e99) does not panic';
+    }
+
+    { # https://rt.cpan.org/Ticket/Display.html?id=79259
+	my $v = $CLASS->new("0.52_0");
+	ok $v->is_alpha, 'Just checking';
+	is $v->numify, '0.520', 'Correctly nummified';
+    }
+
+    { # https://rt.cpan.org/Ticket/Display.html?id=88495
+	@ver::ISA = $CLASS;
+	is ref(ver->new), 'ver', 'ver can inherit from version';
+	is ref(ver->qv("1.2.3")), 'ver', 'ver can inherit from version';
+    }
+
+    { # discovered while integrating with bleadperl
+	eval {my $v = $CLASS->new([1,2,3]) };
+	like $@, qr/Invalid version format/, 'Do not crash for garbage';
+	eval {my $v = $CLASS->new({1 => 2}) };
+	like $@, qr/Invalid version format/, 'Do not crash for garbage';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93603
+	eval {my $v = $CLASS->$method('.1.')};
+	like $@, qr/trailing decimal/, 'Forbid trailing decimals';
+	eval {my $v = $CLASS->$method('.1.2.')};
+	like $@, qr/trailing decimal/, 'Forbid trailing decimals';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93715
+	eval {my $v = $CLASS->new(v1.2)};
+	unlike $@, qr/non-numeric data/, 'Handle short v-strings';
+	eval {my $v = $CLASS->new(v1)};
+	unlike $@, qr/non-numeric data/, 'Handle short v-strings';
+    }
+    {
+	my $two31 = '2147483648';
+	my $v = $CLASS->new($two31);
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+	$v = $CLASS->new("1.$two31.$two31");
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+    }
+    {
+	# now as a number
+	$two31 = 2**31;
+	$v = $CLASS->new($two31);
+	is "$v", 'v.Inf', 'Element Exceeds VERSION_MAX';
+	like $warning, qr/Integer overflow in version/, 'Overflow warning';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=101628
+	undef $warning;
+	$v = $CLASS->new('1.1.00000000010');
+	is $v->normal, "v1.1.10", 'Ignore leading zeros';
+	unlike $warning, qr/Integer overflow in version/, 'No overflow warning';
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=93340
+	$v = $CLASS->parse(q[2.6_01]);
+	is $v->normal, 'v2.601.0', 'Normal strips underscores from alphas'
+    }
+    { # https://rt.cpan.org/Ticket/Display.html?id=98744
+	$v = $CLASS->new("1.02_003");
+	is $v->numify, '1.020030', 'Ignore underscores for numify';
     }
 }
 
 1;
 
-__DATA__
-af_ZA
-af_ZA.utf8
-an_ES
-an_ES.utf8
-az_AZ.utf8
-be_BY
-be_BY.utf8
-bg_BG
-bg_BG.utf8
-br_FR
-br_FR@euro
-br_FR.utf8
-bs_BA
-bs_BA.utf8
-ca_ES
-ca_ES@euro
-ca_ES.utf8
-cs_CZ
-cs_CZ.utf8
-da_DK
-da_DK.utf8
-de_AT
-de_AT@euro
-de_AT.utf8
-de_BE
-de_BE@euro
-de_BE.utf8
-de_DE
-de_DE@euro
-de_DE.utf8
-de_LU
-de_LU@euro
-de_LU.utf8
-el_GR
-el_GR.utf8
-en_DK
-en_DK.utf8
-es_AR
-es_AR.utf8
-es_BO
-es_BO.utf8
-es_CL
-es_CL.utf8
-es_CO
-es_CO.utf8
-es_EC
-es_EC.utf8
-es_ES
-es_ES@euro
-es_ES.utf8
-es_PY
-es_PY.utf8
-es_UY
-es_UY.utf8
-es_VE
-es_VE.utf8
-et_EE
-et_EE.iso885915
-et_EE.utf8
-eu_ES
-eu_ES@euro
-eu_ES.utf8
-fi_FI
-fi_FI@euro
-fi_FI.utf8
-fo_FO
-fo_FO.utf8
-fr_BE
-fr_BE@euro
-fr_BE.utf8
-fr_CA
-fr_CA.utf8
-fr_CH
-fr_CH.utf8
-fr_FR
-fr_FR@euro
-fr_FR.utf8
-fr_LU
-fr_LU@euro
-fr_LU.utf8
-gl_ES
-gl_ES@euro
-gl_ES.utf8
-hr_HR
-hr_HR.utf8
-hu_HU
-hu_HU.utf8
-id_ID
-id_ID.utf8
-is_IS
-is_IS.utf8
-it_CH
-it_CH.utf8
-it_IT
-it_IT@euro
-it_IT.utf8
-ka_GE
-ka_GE.utf8
-kk_KZ
-kk_KZ.utf8
-kl_GL
-kl_GL.utf8
-lt_LT
-lt_LT.utf8
-lv_LV
-lv_LV.utf8
-mk_MK
-mk_MK.utf8
-mn_MN
-mn_MN.utf8
-nb_NO
-nb_NO.utf8
-nl_BE
-nl_BE@euro
-nl_BE.utf8
-nl_NL
-nl_NL@euro
-nl_NL.utf8
-nn_NO
-nn_NO.utf8
-no_NO
-no_NO.utf8
-oc_FR
-oc_FR.utf8
-pl_PL
-pl_PL.utf8
-pt_BR
-pt_BR.utf8
-pt_PT
-pt_PT@euro
-pt_PT.utf8
-ro_RO
-ro_RO.utf8
-ru_RU
-ru_RU.koi8r
-ru_RU.utf8
-ru_UA
-ru_UA.utf8
-se_NO
-se_NO.utf8
-sh_YU
-sh_YU.utf8
-sk_SK
-sk_SK.utf8
-sl_SI
-sl_SI.utf8
-sq_AL
-sq_AL.utf8
-sr_CS
-sr_CS.utf8
-sv_FI
-sv_FI@euro
-sv_FI.utf8
-sv_SE
-sv_SE.iso885915
-sv_SE.utf8
-tg_TJ
-tg_TJ.utf8
-tr_TR
-tr_TR.utf8
-tt_RU.utf8
-uk_UA
-uk_UA.utf8
-vi_VN
-vi_VN.tcvn
-wa_BE
-wa_BE@euro
-wa_BE.utf8
-
